%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [A,P,var]=cnst_grw(F,var,mat)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determine tangent, stress and internal variable 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% input:  F     = [ 2 x 2 ]            ... deformation gradient F
%         var   = [ 4 ]                ... internal variables / density
%         mat   = [ ... ]              ... material parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% output: A     = [ 2 x 2 x 2 x 2 ]    ... material tangent operator
%         P     = [ 2 x 2 ]            ... 1st piola kirchhoff stress
%         var   = [ 4 ]                ... internal variables / density
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
emod = mat(1);   nue  = mat(2);   rho0 = mat(3);   psi0 = mat(4);   
expm = mat(5);   expn = mat(6);   dt   = mat(7);

xmu = emod / 2.0 / (1.0+nue);
xlm = emod * nue / (1.0+nue) / ( 1.0-2.0*nue ); 

F_inv = inv(F);
J     = det(F);
delta = [1 0; 0 1];

%%% update internal variable %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[var,facs,fact]=upd_dens(F,var,mat);
  
%%% first piola kirchhoff stress %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
P =  facs * (xmu * F + (xlm * log(J) - xmu) * F_inv'); 
   
%%% tangent %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:2
for j=1:2
for k=1:2
for l=1:2
  A(i,j,k,l) =  xlm                 * F_inv(j,i)*F_inv(l,k) ...
			 - (xlm * log(J) - xmu) * F_inv(l,i)*F_inv(j,k) ...
			 +                 xmu  * delta(i,k)*delta(j,l);
  A(i,j,k,l) = facs * A(i,j,k,l)    +  fact * P(i,j)*P(k,l);			 
end, end, end, end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
